<?php

namespace IMATHUZH\Qfq\Core\Form\FormElement;

use IMATHUZH\Qfq\Core\Form\Form;
use IMATHUZH\Qfq\Core\Helper\HelperFormElement;
use IMATHUZH\Qfq\Core\Helper\Path;
use IMATHUZH\Qfq\Core\Helper\Support;

class EditorFormElement extends AbstractFormElement
{
    public array $htmlAttributesCharacterCount;

    public function __construct($attributes, ?Form $form = null)
    {
        parent::__construct($attributes, $form);

        if (empty($this->attributes[FE_EDITOR_TYPE])) {
            $this->attributes[FE_EDITOR_TYPE] = FE_EDITOR_TYPE_TINYMCE;
        }

        $this->attributes[FE_TMP_EXTRA_BUTTON_HTML] = $this->attributes[FE_TMP_EXTRA_BUTTON_HTML] ?? '';
        $this->attributes[FE_INPUT_EXTRA_BUTTON_INFO] = $this->attributes[FE_INPUT_EXTRA_BUTTON_INFO] ?? '';

        switch ($this->attributes[FE_EDITOR_TYPE]) {
            case FE_EDITOR_TYPE_TINYMCE:
                $this->handleTinyMCE();
                break;
            case FE_EDITOR_TYPE_CODEMIRROR:
                $this->handleCodeMirror();
                break;
            default:
                throw new \UserFormException("Unexpected editor type: '" . $this->attributes[FE_EDITOR_TYPE] . "'", ERROR_FORMELEMENT_EDITOR_TYPE);
        }
    }


    private function handleCodeMirror(){
        $this->htmlAttributes[ATTRIBUTE_DATA_REFERENCE] = $this->attributes[FE_DATA_REFERENCE];
        $this->htmlAttributes[FE_DATA_LOAD] = $this->attributes[FE_DYNAMIC_UPDATE] == 'yes' ? FE_DATA_LOAD : '';
        array_push($this->cssClasses, 'qfq-codemirror');

        // Defaults: fits good to QFQ style programming
        $this->attributes[FE_EDITOR_PREFIX . 'mode'] = $this->attributes[FE_EDITOR_PREFIX . 'mode'] ?? "qfq";
        $this->attributes[FE_EDITOR_PREFIX . 'lineNumbers'] = $this->attributes[FE_EDITOR_PREFIX . 'lineNumbers'] ?? true;
        $this->attributes[FE_EDITOR_PREFIX . 'lineWrapping'] = $this->attributes[FE_EDITOR_PREFIX . 'lineWrapping'] ?? true;
        $this->attributes[FE_EDITOR_PREFIX . 'tabSize'] = $this->attributes[FE_EDITOR_PREFIX . 'tabSize'] ?? 2;

        if ($this->attributes[FE_MODE] == FE_MODE_READONLY) {
            $this->attributes[FE_EDITOR_PREFIX . 'readOnly'] = true;
        }

        //$json = $this->getPrefixedElementsAsJSON(FE_EDITOR_PREFIX, $formElement);
        //$attribute .= Support::doAttribute('data-config', $json, true);

        $minMax = explode(',', $this->attributes[FE_SIZE], 2);
        if (isset($minMax[0]) && $minMax[0] !== '') {
            $this->htmlAttributes['data-height'] = $minMax[0];
        }

        //$attribute .= HelperFormElement::getAttributeList($formElement, [F_FE_DATA_PATTERN_ERROR, F_FE_DATA_REQUIRED_ERROR, F_FE_DATA_MATCH_ERROR, F_FE_DATA_ERROR]);

        //$wrapSetupClass = $this->wrap[WRAP_SETUP_ELEMENT][WRAP_SETUP_CLASS] ?? '';
        //$json = $this->getFormElementForJson($htmlFormElementName, $value, $formElement, $wrapSetupClass);
    }


    private function handleTinyMCE()
    {
        $attribute = '';
        array_push($this->cssClasses, 'qfq-tinymce');
        $this->htmlAttributes['data-control-name'] = $this->htmlAttributes[HTML_ATTR_NAME];
        $this->htmlAttributes['data_placeholder'] = $this->attributes[FE_PLACEHOLDER];
        $this->htmlAttributes[FE_DATA_LOAD] = $this->attributes[FE_DYNAMIC_UPDATE] === 'yes' ? FE_DATA_LOAD : '';
        $this->htmlAttributes['data-title'] = $this->attributes[FE_TOOLTIP];
        $this->setEditorTinyMCEConfig();

        // Get standard upload filepath from user and convert to SIP
        if (isset($this->attributes[FE_EDITOR_FILE_UPLOAD_PATH]) && $this->attributes[FE_EDITOR_FILE_UPLOAD_PATH] != '') {
            $base64Param = base64_encode($this->attributes[FE_EDITOR_FILE_UPLOAD_PATH]);
            $urlParam = 'fileUploadPath=' . $base64Param;
            $sipParam = $this->sip->queryStringToSip($urlParam, RETURN_SIP);
            $completeUrl = '?action=imageUpload&s=' . $sipParam;
        } else {
            $completeUrl = '?action=imageUpload';
        }

        // Prepare maxLength and characterCount for tinyMce
        $maxLength = $this->attributes[FE_MAX_LENGTH];
        $elementCharacterCount = '';
        if (isset($this->attributes[FE_CHARACTER_COUNT_WRAP])) {
            $this->htmlAttributes['data-character-count-id'] = $this->attributes[FE_HTML_ID] . HTML_ID_EXTENSION_CHARACTER_COUNT;
            $this->htmlAttributesCharacterCount = array();
            $this->htmlAttributesCharacterCount[HTML_ATTR_ID] = $this->attributes[FE_HTML_ID] . HTML_ID_EXTENSION_CHARACTER_COUNT;
            $this->htmlAttributesCharacterCount['class'] = $this->attributes[FE_CHARACTER_COUNT_WRAP] == '' ? 'qfq-cc-style' : '';
        }

        // Get baseUrl for image upload. Path handling in T3 => V10 needs absolute path.
        $baseUrl = $this->store::getVar(SYSTEM_BASE_URL, STORE_SYSTEM);

        //TODO: static setup for TinyMCE ImagePlugin - needs to be activated / dynamically set by QFQ parameter.
        $preSettings = [
#            "plugins" => "image",
            "plugins" => "charmap",
            "maxLength" => $maxLength,
            "file_picker_types" => "file image media",
            "image_advtab" => true,
            "automatic_uploads" => true,
            "images_upload_url" => $baseUrl . Path::appToApi(API_FILE_PHP) . $completeUrl,
            "images_reuse_filename" => true,
            "paste_data_images" => true
        ];

        // TODO: JSON Stuff
        //$json = $this->getPrefixedElementsAsJSON(FE_EDITOR_PREFIX, $formElement, $preSettings);
        //$attribute .= Support::doAttribute('data-config', $json, true);

        //$attribute .= HelperFormElement::getAttributeList($formElement, [F_FE_DATA_PATTERN_ERROR, F_FE_DATA_REQUIRED_ERROR, F_FE_DATA_MATCH_ERROR, F_FE_DATA_ERROR]);

        //$wrapSetupClass = $this->wrap[WRAP_SETUP_ELEMENT][WRAP_SETUP_CLASS] ?? '';
        //$json = $this->getFormElementForJson($htmlFormElementName, $value, $formElement, $wrapSetupClass);
    }


    /**
     * Parse $formElement[FE_EDITOR_*] settings and build editor settings.
     */
    private function setEditorTinyMCEConfig() {
        // plugins
        if (!isset($this->attributes[FE_EDITOR_PREFIX . 'plugins'])) {
            $this->attributes[FE_EDITOR_PREFIX . 'plugins'] = 'code link lists searchreplace table textcolor textpattern visualchars image,paste';
        }

        // toolbar: https://www.tinymce.com/docs/advanced/editor-control-identifiers/#toolbarcontrols
        if (!isset($this->attributes[FE_EDITOR_PREFIX . 'toolbar'])) {
            $this->attributes[FE_EDITOR_PREFIX . 'toolbar'] = 'code searchreplace undo redo | ' .
                'styleselect link table | ' .
                'bullist numlist outdent indent | forecolor backcolor bold italic';
        }

        // menubar
        if (!isset($this->attributes[FE_EDITOR_PREFIX . 'menubar'])) {
            $this->attributes[FE_EDITOR_PREFIX . 'menubar'] = 'false';
        }

        // autofocus
        if (isset($this->attributes[FE_AUTOFOCUS]) && $this->attributes[FE_AUTOFOCUS] == 'yes') {
            $this->attributes[FE_EDITOR_PREFIX . 'auto_focus'] = $this->htmlAttributes[HTML_ATTR_NAME];
        }

        // valid elements
        // Set defaults for tinyMce
        $imgToken = 'img[longdesc|usemap|src|border|alt=|title|hspace|vspace|width|height|align]';
        $textDecoration = 'span[style]';
        $table = 'table[style|align|border],td[style],th[style],tr[style],tbody[style],thead[style]';
        $url = 'a[href|target|title]';
        $paragraphToken = 'p[align]';
        $strong = 'strong';

        $htmlAllowArray = array();

        if (isset($this->attributes[FE_HTML_ALLOW])) {
            $htmlAllowArray = explode(',', $this->attributes[FE_HTML_ALLOW]);
        }

        $formatDropdownElements = array();
        $customEditorToolbar = 'code |';

        // flags to prevent multiple same values
        $listFlag = false;
        $decorationFlag = false;
        $formatDropdownFlag = false;
        $customEditorToolbarFlags = array();

        foreach ($htmlAllowArray as $htmlToken => $value) {
            switch ($value) {
                case 'a':
                    $htmlAllowArray[$htmlToken] = $url;
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'link');
                    break;
                case 'table':
                    $htmlAllowArray[$htmlToken] = $table;
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'table');
                    break;
                case 'textDecoration':
                case 'u':
                case 'ins':
                case 's':
                case 'del':
                    if (!$decorationFlag) {
                        $htmlAllowArray[$htmlToken] = $textDecoration;
                        $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'textDecoration', 'underline strikethrough');
                        $decorationFlag = true;
                    }
                    break;
                case 'img':
                    $htmlAllowArray[$htmlToken] = $imgToken;
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'image');
                    break;
                case 'ul':
                    if (!$listFlag) {
                        $htmlAllowArray[$htmlToken] = 'ul,li';
                        $listFlag = true;
                    } else {
                        $htmlAllowArray[$htmlToken] = 'ul';
                    }
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'bullist');
                    break;
                case 'ol':
                    if (!$listFlag) {
                        $htmlAllowArray[$htmlToken] = 'ol,li';
                        $listFlag = true;
                    } else {
                        $htmlAllowArray[$htmlToken] = 'ol';
                    }
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'numlist');
                    break;
                case 'b':
                case 'strong':
                    $htmlAllowArray[$htmlToken] = $strong;
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'bold');
                    break;
                case 'i':
                case 'em':
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'italic');
                    break;
                case 'sub':
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'subscript');
                    break;
                case 'sup':
                    $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'supscript');
                    break;
                case 'h1':
                case 'h2':
                case 'h3':
                case 'h4':
                case 'h5':
                case 'h6':
                case 'p':
                case 'div':
                case 'pre':
                    if ($value === 'p') {
                        $htmlAllowArray[$htmlToken] = $paragraphToken;
                    }
                    $this->setTinymceBlockFormats($value, $formatDropdownElements);
                    $formatDropdownFlag = true;
                    break;
                default:
                    break;
            }
        }

        // set format dropdown at the end of the toolbar if its used
        if ($formatDropdownFlag) {
            $this->setTinymceEditorToolbarAttributes($customEditorToolbarFlags, $customEditorToolbar, 'formatselect');
        }

        // If htmlAllow is used: toolbar will be overwritten with the customized one
        if (!empty($customEditorToolbarFlags)) {
            $this->attributes[FE_EDITOR_PREFIX . 'toolbar'] = $customEditorToolbar;
        }

        // Set allowed values and corrected dropdown from formats
        if (isset($this->attributes[FE_HTML_ALLOW])) {
            $this->attributes[FE_EDITOR_PREFIX . 'valid_elements'] = implode(',', $htmlAllowArray);
            $this->attributes[FE_TINYMCE_DROPDOWN_FORMATS] = implode(';', $formatDropdownElements);
            $this->attributes[FE_EDITOR_PREFIX . 'block_formats'] = $this->attributes[FE_TINYMCE_DROPDOWN_FORMATS];
        }

        // Check for min_height, max_height
        $minMax = explode(',', $this->attributes[FE_SIZE], 2);
        if (isset($minMax[0]) && ctype_digit($minMax[0]) && !isset($this->attributes[FE_EDITOR_PREFIX . 'min_height'])) {
            $this->attributes[FE_EDITOR_PREFIX . 'min_height'] = $minMax[0];
        }
        if (isset($minMax[1]) && ctype_digit($minMax[1]) && !isset($this->attributes[FE_EDITOR_PREFIX . 'max_height'])) {
            $this->attributes[FE_EDITOR_PREFIX . 'max_height'] = $minMax[1];
        }

        return $this->attributes;
    }


    /**
     * Tinymce toolbar attributes needs to be defined separately
     *
     * @param $toolbarFlags
     * @param $attributeList
     * @param $attributeName
     * @param $specialAttributeName
     * @return void
     */
    private function setTinymceEditorToolbarAttributes(&$toolbarFlags, &$attributeList, $attributeName, $specialAttributeName = '') {
        if (!empty($toolbarFlags[$attributeName]) && !$toolbarFlags[$attributeName]) {
            if ($specialAttributeName === '') {
                $attributeList .= ' ' . $attributeName;
            } else {
                $attributeList .= ' ' . $specialAttributeName;
            }
            $toolbarFlags[$attributeName] = true;
        }
    }


    /**
     * Tinymce has individual strings for some html block configurations. They need to be set here.
     *
     * @param $format
     * @param $formatDropdownElements
     * @return void
     */
    private function setTinymceBlockFormats($format, &$formatDropdownElements) {

        switch ($format) {
            case 'h1':
                $formatDropdownElements[] = 'Heading 1=h1';
                break;
            case 'h2':
                $formatDropdownElements[] = 'Heading 2=h2';
                break;
            case 'h3':
                $formatDropdownElements[] = 'Heading 3=h3';
                break;
            case 'h4':
                $formatDropdownElements[] = 'Heading 4=h4';
                break;
            case 'h5':
                $formatDropdownElements[] = 'Heading 5=h5';
                break;
            case 'h6':
                $formatDropdownElements[] = 'Heading 6=h6';
                break;
            case 'p':
                $formatDropdownElements[] = 'Paragraph=p';
                break;
            case 'div':
                $formatDropdownElements[] = 'Div-Container=div';
                break;
            case 'pre':
                $formatDropdownElements[] = 'Preformat=pre';
                break;
            default:
                break;
        }
    }
}