<?php

namespace IMATHUZH\Qfq\Core\Renderer\FormElement;

use IMATHUZH\Qfq\Core\Form\FormElement\AbstractFormElement;
use IMATHUZH\Qfq\Core\Helper\HelperFormElement;
use IMATHUZH\Qfq\Core\Helper\Support;

abstract class NativeRenderer extends FormElementRenderer
{
    /**
     * @var array
     * Defines the html tags that wrap label, input and note. This is initialized in $this->wrapLabelInputNote
     */
    public array $wrap = array();


    public function renderFormElement(AbstractFormElement $fe, string $renderMode = RENDER_MODE_HTML) : string{

        if($fe->attributes[FE_MODE] == FE_MODE_HIDDEN) return "";

        // Setup Wrap elements for label, input and note
        $this->wrapLabelInputNote($fe->attributes[FE_BS_LABEL_COLUMNS], $fe->attributes[FE_BS_INPUT_COLUMNS], $fe->attributes[FE_BS_NOTE_COLUMNS]);
        $rowId = $fe->htmlAttributes[HTML_ATTR_ID] . "-r";

        return  $fe->attributes[FE_HTML_BEFORE]
            . ($fe->attributes[FE_FLAG_ROW_OPEN_TAG] == 1 ? '<div id="' . $rowId . '" class="form-group clearfix">' : '')
            . $this->wrap[WRAP_SETUP_LABEL][WRAP_SETUP_START]
            . $this->renderLabel($fe, $renderMode)
            . $this->wrap[WRAP_SETUP_LABEL][WRAP_SETUP_END]
            . $this->wrap[WRAP_SETUP_INPUT][WRAP_SETUP_START]
            . $this->renderInput($fe, $renderMode)
            . $this->renderFormEditorUrl($fe)
            . $this->wrap[WRAP_SETUP_INPUT][WRAP_SETUP_END]
            . $this->wrap[WRAP_SETUP_NOTE][WRAP_SETUP_START]
            . $this->renderNote($fe, $renderMode)
            . $this->wrap[WRAP_SETUP_NOTE][WRAP_SETUP_END]
            . ($fe->attributes[FE_FLAG_ROW_CLOSE_TAG] == 1 ? '</div>' : '')
            . $fe->attributes[FE_HTML_AFTER];
    }

    /**
     * @param AbstractFormElement $fe
     * @param string $renderMode
     * @return string
     * Renders the label part of the formElement
     */
    protected function renderLabel(AbstractFormElement $fe, string $renderMode = RENDER_MODE_HTML) : string{
        $additionalCssClasses = is_array($fe->addClassRequired) ? implode(' ', $fe->addClassRequired) : $fe->addClassRequired;
        return HelperFormElement::buildLabel($fe->htmlAttributes[HTML_ATTR_NAME], $fe->attributes[FE_LABEL], $additionalCssClasses);
    }

    /**
     * @param AbstractFormElement $fe
     * @param string $renderMode
     * @return void
     * Renders only the Input element. No label, note or additional wrappers around the input is rendered.
     * This function is public, so that it can be used to create inline-edit elements.
     */
    public abstract function renderInput(AbstractFormElement $fe, string $renderMode = RENDER_MODE_HTML) : string;

    /**
     * @param AbstractFormElement $fe
     * @param string $renderMode
     * @return void
     * Renders the note part of the formElement.
     */
    protected function renderNote(AbstractFormElement $fe, string $renderMode = RENDER_MODE_HTML) : string{
        if (!empty($fe->addClassRequired[FE_NOTE])) {
            $fe->attributes[FE_NOTE] = Support::wrapTag('<span class="' . $fe->addClassRequired[FE_NOTE] . '">', $fe->attributes[FE_NOTE]);
        }
        return $fe->attributes[FE_NOTE];
        //return $this->customWrap($fe->attributes, $fe->attributes[FE_NOTE], FE_WRAP_NOTE, $fe->attributes[FE_BS_NOTE_COLUMNS],
        //    [$this->wrap[WRAP_SETUP_NOTE][WRAP_SETUP_START], $this->wrap[WRAP_SETUP_NOTE][WRAP_SETUP_END]], $fe->attributes[FE_HTML_ID] . HTML_ID_EXTENSION_NOTE);

    }


    /**
     * Fill the BS wrapper for Label/Input/Note.
     * For legacy reasons, $label/$input/$note might be a number (0-12) or the bs column classes ('col-md-12 col-lg-9')
     * If a Renderer is implemented which does not use the BS column system, then this functionality will have to be moved elsewhere.
     *
     * @param $label
     * @param $input
     * @param $note
     */
    protected function wrapLabelInputNote($label, $input, $note) : string {
        $output = "";

        $label = is_numeric($label) ? ('col-md-' . $label) : $label;
        $this->wrap[WRAP_SETUP_LABEL][WRAP_SETUP_START] = "<div class='$label qfq-label'>";
        $this->wrap[WRAP_SETUP_LABEL][WRAP_SETUP_END] = "</div>";

        $input = is_numeric($input) ? ('col-md-' . $input) : $input;
        $this->wrap[WRAP_SETUP_INPUT][WRAP_SETUP_START] = "<div class='$input'>";
        $this->wrap[WRAP_SETUP_INPUT][WRAP_SETUP_END] = "</div>";

        $note = is_numeric($note) ? ('col-md-' . $note) : $note;
        $this->wrap[WRAP_SETUP_NOTE][WRAP_SETUP_START] = "<div class='$note qfq-note'>";
        $this->wrap[WRAP_SETUP_NOTE][WRAP_SETUP_END] = "</div>";

        return $output;
    }

    /**
     * Wrap content with $wrapArray or, if specified use $formElement[$wrapName]. Inject $htmlId in wrap.
     *
     * Result:
     * - if $bsColumns==0 and empty $formElement[$wrapName]: no wrap
     * - if $formElement[$wrapName] is given: wrap with that one. Else: wrap with $wrapArray
     * - if $htmlId is give, inject it in $wrap.
     *
     * @param array $formElement Complete FormElement, especially some FE_WRAP
     * @param string $htmlElement Content to wrap.
     * @param string $wrapName FE_WRAP_ROW, FE_WRAP_LABEL, FE_WRAP_INPUT, FE_WRAP_NOTE
     * @param int $bsColumns
     * @param array $wrapArray System wide Defaults: [ 'open wrap', 'close wrap' ]
     * @param string $htmlId
     * @param string $class
     *
     * @return string Wrapped $htmlElement
     * @throws \CodeException
     * @throws \UserFormException
     */
    protected function customWrap(array $formElement, $htmlElement, $wrapName, $bsColumns, array $wrapArray, $htmlId = '', $class = '') {

        // If $bsColumns==0: do not wrap with default.
        if ($bsColumns == '0') {
            $wrapArray[0] = '';
            $wrapArray[1] = '';
        }

        // If there is a 'per FormElement'-wrap, take it.
        if (isset($formElement[$wrapName])) {
            $wrapArray = explode('|', $formElement[$wrapName], 2);
        }

        if (count($wrapArray) != 2) {
            throw new \UserFormException("Need open & close wrap token for FormElement.parameter" . $wrapName . " - E.g.: <div ...>|</div>", ERROR_MISSING_VALUE);
        }

        if ($wrapArray[0] != '') {
            $wrapArray[0] = Support::insertAttribute($wrapArray[0], 'id', $htmlId);
            $wrapArray[0] = Support::insertAttribute($wrapArray[0], 'class', $class); // might be problematic, if there is already a 'class' defined.
            if ($wrapName == FE_WRAP_LABEL) {
                $wrapArray[0] = Support::insertAttribute($wrapArray[0], 'style', 'text-align: ' . $formElement[F_FE_LABEL_ALIGN] . ';'); // might be problematic, if there is already a 'class' defined.
            }
            // Insert 'required="required"' for checkboxes and radios
            if ($wrapName == FE_WRAP_INPUT && $formElement[FE_MODE] == FE_MODE_REQUIRED &&
                ($formElement[FE_TYPE] == FE_TYPE_CHECKBOX || $formElement[FE_TYPE] == FE_TYPE_RADIO)) {
                $wrapArray[0] = Support::insertAttribute($wrapArray[0], 'required', 'required'); // might be problematic, if there is already a 'class' defined.
            }
        }

        return $wrapArray[0] . $htmlElement . $wrapArray[1];
    }

}